#include <stdio.h>
#include <errno.h>

#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_temporaly_array.h"

#include "tt_bmp_image.h"


const int TtBmpImage::OLD_CLOSE   = 32767;
const int TtBmpImage::SIZE        = 32766;
const int TtBmpImage::OLD_UPARROW = 32765;
const int TtBmpImage::OLD_DNARROW = 32764;
const int TtBmpImage::OLD_RGARROW = 32763;
const int TtBmpImage::OLD_LFARROW = 32762;
const int TtBmpImage::BTSIZE      = 32761;
const int TtBmpImage::CHECK       = 32760;
const int TtBmpImage::CHECKBOXES  = 32759;
const int TtBmpImage::BTNCORNERS  = 32758;
const int TtBmpImage::OLD_REDUCE  = 32757;
const int TtBmpImage::OLD_ZOOM    = 32756;
const int TtBmpImage::OLD_RESTORE = 32755;
const int TtBmpImage::CLOSE       = 32754;
const int TtBmpImage::UPARROW     = 32753;
const int TtBmpImage::DNARROW     = 32752;
const int TtBmpImage::RGARROW     = 32751;
const int TtBmpImage::LFARROW     = 32750;
const int TtBmpImage::REDUCE      = 32749;
const int TtBmpImage::ZOOM        = 32748;
const int TtBmpImage::RESTORE     = 32747;
const int TtBmpImage::REDUCED     = 32746;
const int TtBmpImage::ZOOMD       = 32745;
const int TtBmpImage::RESTORED    = 32744;
const int TtBmpImage::UPARROWD    = 32743;
const int TtBmpImage::DNARROWD    = 32742;
const int TtBmpImage::RGARROWD    = 32741;
const int TtBmpImage::LFARROWD    = 32740;
const int TtBmpImage::MNARROW     = 32739;
const int TtBmpImage::COMBO       = 32738;
const int TtBmpImage::UPARROWI    = 32737;
const int TtBmpImage::DNARROWI    = 32736;
const int TtBmpImage::RGARROWI    = 32735;
const int TtBmpImage::LFARROWI    = 32734;


TtBmpImage::TtBmpImage( int id )
{
  handle_ = ::LoadBitmap( NULL, (const char *)id );
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::LoadBitmap ) );
  }
}

TtBmpImage::TtBmpImage( const std::string& path )
{
  size_t readed_size;

  FILE* file = fopen( path.c_str(), "rb" );
  if ( file == NULL ) {
    throw TtFileOpenException( path, errno );
  }

  BITMAPFILEHEADER file_header;
  readed_size = fread( reinterpret_cast<char*>( &file_header ), 1, sizeof( file_header ), file );
  if ( readed_size < sizeof( file_header ) ) {
    throw TT_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( fread ), errno );
  }

  // t@C̐擪 BM ł͖ꍇ
  if ( file_header.bfType != 0x4d42 ) {
    throw TtInvalidFileTypeException( typeid( *this ), "Bitmap" );
  }

  TtTemporalyString buf( file_header.bfSize - sizeof( file_header ) );
  readed_size = fread( buf.GetPtr(), 1, buf.GetSize(), file );
  if ( readed_size < buf.GetSize() ) {
    throw TT_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( fread ), errno );
  }

  BITMAPINFOHEADER* info_header = reinterpret_cast<BITMAPINFOHEADER*>( buf.GetPtr() );
  handle_ = ::CreateBitmap( info_header->biWidth,
                            info_header->biHeight,
                            info_header->biPlanes,
                            info_header->biBitCount,
                            buf.GetPtr() + ( file_header.bfOffBits - sizeof( BITMAPFILEHEADER ) ) );
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::CreateBitmap ) );
  }

  fclose( file );
  return;

}

TtBmpImage::TtBmpImage( HINSTANCE h_instance, const std::string& class_name )
{
  handle_ = ::LoadBitmap( h_instance, class_name.c_str() );
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::LoadBitmap ) );
  }
}

TtBmpImage::~TtBmpImage()
{
  if ( handle_ != NULL ) {
    DeleteObject( handle_ );
  }
}

HBITMAP
TtBmpImage::GetHandle( void ) const
{
  return handle_;
}

int
TtBmpImage::GetWidth( void ) const
{
  BITMAP tmp;
  if ( ::GetObject( handle_, sizeof( BITMAP ), &tmp ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::GetObject ) );
  }
  return tmp.bmWidth;
}

int
TtBmpImage::GetHeight( void ) const
{
  BITMAP tmp;
  if ( ::GetObject( handle_, sizeof( BITMAP ), &tmp ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::GetObject ) );
  }
  return tmp.bmHeight;
}
