#include "ttl_define.h"
#include "tt_exception.h"

#include "tt_dialog.h"


BOOL CALLBACK
TtDialog::WindowProcedureForTTLDialog( HWND handle, UINT msg, WPARAM w_param, LPARAM l_param )
{
  if ( msg == WM_INITDIALOG ) {
    TtDialog& dialog = *(reinterpret_cast<TtDialog*>( l_param ));
    dialog.handle_ = handle;
    dialog.SetFont( TtFont::DEFAULT );
    TtWindow::GetTtWindowTable().insert( std::pair<HWND, TtWindow*>( handle, &dialog ) );
    if ( NOT( dialog.Created() ) ) {
      ::DestroyWindow( dialog.handle_ );
      return FALSE;
    }
  }
  TtWindow* window = TtWindow::FindWindow( handle );
  return ( window != NULL ) ? window->WindowProcedure( msg, w_param, l_param ) : FALSE;
}



DWORD
TtDialog::GetStyle( void )
{
  return WS_DLGFRAME | WS_SYSMENU;
}

DWORD
TtDialog::GetExtendedStyle( void )
{
  return 0;
}

bool
TtDialog::Created( void )
{
  return true;
}

LRESULT
TtDialog::DefaultWindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param )
{
  NOT_USE_VAR( msg );
  NOT_USE_VAR( w_param );
  NOT_USE_VAR( l_param );
  return FALSE;
}

LRESULT
TtDialog::WMCloseReceived( void )
{
  this->EndDialog( 0 );
  return 0;
}

LRESULT
TtDialog::WMDestroyReceived( void )
{
  return FALSE;
}

LRESULT
TtDialog::WindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param )
{
  LRESULT result = this->TtForm::WindowProcedure( msg, w_param, l_param );
  return ( result == 0 || result == TtWindow::MSG_DEFAULT ) ? FALSE : TRUE;
}


TtDialog::TtDialog( void ) :
TtForm(),
template_object_( GlobalAlloc( GMEM_FIXED | GMEM_ZEROINIT, sizeof( DLGTEMPLATE ) + 10 ) ),
dialog_result_( 0 )
{
}

TtDialog::~TtDialog()
{
  GlobalFree( template_object_ );
}

void
TtDialog::CreatePrimitive( CreateParameter& parameter )
{
  NOT_USE_VAR( parameter );
  throw TtInvalidOperationException( typeid( *this ) );
}

int
TtDialog::ShowDialog( TtForm& parent )
{
  parent_ = &parent;
  instance_handle_ = parent.GetInstanceHandle();
  id_ = 0;

  DLGTEMPLATE& tmp = *(static_cast<DLGTEMPLATE*>( template_object_ ) );
  tmp.style = this->GetStyle();
  tmp.dwExtendedStyle = this->GetExtendedStyle();
  tmp.cdit = 0;
  tmp.x = 100;
  tmp.y = 100;
  tmp.cx = 100;
  tmp.cy = 100;
  int ret = ::DialogBoxIndirectParam(
    instance_handle_, &tmp, parent.GetHandle(),
    TtDialog::WindowProcedureForTTLDialog, reinterpret_cast<LPARAM>( this ) );
  if ( ret == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::DialogBoxIndirectParam ) );
  }
  if ( handle_ != NULL ) {
    TtWindow::GetTtWindowTable().erase( handle_ );
    handle_ = NULL;
  }
  return ret;
}


void
TtDialog::EndDialog( int result )
{
  if ( ::EndDialog( handle_, result ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::EndDialog ) );
  };
  dialog_result_ = result;
}


int
TtDialog::GetResult( void )
{
  return dialog_result_;
}

// -- TtDialogModeless ---------------------------------------------------
TtDialogModeless::TtDialogModeless( void ) :
TtDialog()
{
}

TtDialogModeless::~TtDialogModeless()
{
}


int
TtDialogModeless::ShowDialog( TtForm& parent )
{
  parent_ = &parent;
  instance_handle_ = parent.GetInstanceHandle();
  id_ = 0;

  DLGTEMPLATE& tmp = *(static_cast<DLGTEMPLATE*>( template_object_ ) );
  tmp.style = this->GetStyle();
  tmp.dwExtendedStyle = this->GetExtendedStyle();
  tmp.cdit = 0;
  tmp.x = 100;
  tmp.y = 100;
  tmp.cx = 100;
  tmp.cy = 100;
  handle_ = ::CreateDialogIndirectParam(
    instance_handle_, &tmp, parent.GetHandle(),
    TtDialog::WindowProcedureForTTLDialog, reinterpret_cast<LPARAM>( this ) );
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::CreateDialogIndirectParam ) );
  }
  return 0;
}

void
TtDialogModeless::EndDialog( int result )
{
  if ( ::DestroyWindow( handle_ ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::DestroyWindow ) );
  }
  handle_ = NULL;
  dialog_result_ = result;
}
