#include <errno.h>

#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_path.h"

#include "tt_directory.h"


namespace {
  class FindCloser {
  public:
    explicit FindCloser( HANDLE handle ) : handle_( handle ) {}
    ~FindCloser() {
      if ( handle_ != INVALID_HANDLE_VALUE ) {
        FindClose( handle_ );
      }
    }
  private:
    HANDLE handle_;
  };
}


TtDirectory::TtDirectory( const std::string& path ) :
self_path_( path )
{
  if ( NOT( TtPath::IsDirectory( path ) ) ) {
    throw TtFileOpenException( path, ENOENT );
  }
}

const std::string&
TtDirectory::GetPath( void ) const
{
  return self_path_;
}


static void
get_entries( const std::string&             self_path,
             const std::string&             path,
             std::vector<std::string>&      array,
             TtDirectory::FilteringFunction filter )
{
  WIN32_FIND_DATA find_data;
  std::string search_path = self_path + "\\" + path;

  HANDLE search_handle = ::FindFirstFile( search_path.c_str(), &find_data );
  if ( search_handle == INVALID_HANDLE_VALUE ) {
    if ( ::GetLastError() != ERROR_FILE_NOT_FOUND && ::GetLastError() != ERROR_ACCESS_DENIED ) {
      throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::FindFirstFile ) );
    }
    return;
  }
  FindCloser closer( search_handle );

  do {
    std::string tmp = self_path + "\\" + find_data.cFileName;
    if ( filter == NULL || (*filter)( tmp ) ) {
      array.push_back( tmp );
    }
  }
  while ( ::FindNextFile( search_handle, &find_data ) != 0 );

  if ( ::GetLastError() != ERROR_NO_MORE_FILES ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::FindNextFile ) );
  }
}



std::vector<std::string>
TtDirectory::GetEntries( const std::string& path, FilteringFunction filter ) const
{
  std::vector<std::string> array;
  get_entries( self_path_, path, array, filter );
  return array;
}


static void
recursive( const std::string&             self_path,
           const std::string&             path,
           std::vector<std::string>&      array,
           TtDirectory::FilteringFunction filter )
{
  get_entries( self_path, path, array, filter );

  WIN32_FIND_DATA find_data;
  std::string search_path = self_path + "\\*";

  HANDLE search_handle = ::FindFirstFileEx(
    search_path.c_str(), FindExInfoStandard, &find_data, FindExSearchLimitToDirectories, NULL, 0 );
  if ( search_handle == INVALID_HANDLE_VALUE ) {
    if ( ::GetLastError() != ERROR_FILE_NOT_FOUND && ::GetLastError() != ERROR_ACCESS_DENIED ) {
      throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::FindFirstFileEx ) );
    }
    return;
  }
  FindCloser closer( search_handle );

  do {
    if ( (find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
         strcmp( find_data.cFileName, "." ) != 0 && strcmp( find_data.cFileName, ".." ) ) {
      recursive( self_path + "\\" + find_data.cFileName, path, array, filter );
    }
  }
  while ( ::FindNextFile( search_handle, &find_data ) != 0 );

  if ( ::GetLastError() != ERROR_NO_MORE_FILES ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::FindNextFile ) );
  }
}


std::vector<std::string>
TtDirectory::GetEntriesRecursive( const std::string& path, FilteringFunction filter ) const
{
  std::vector<std::string> array;
  recursive( self_path_, path, array, filter );
  return array;
}
