#include <windows.h>
#include <dbghelp.h>

#include "ttl_define.h"

#include "tt_dynamic_link_library.h"


TtDynamicLinkLibrary::TtDynamicLinkLibrary( const std::string& filename ) :
handle_( ::LoadLibrary( filename.c_str() ) ),
base_( reinterpret_cast<LPBYTE>( handle_ ) ),
function_name_list_()
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::LoadLibrary ) );
  }

  unsigned long size;
  IMAGE_EXPORT_DIRECTORY* directory = static_cast<IMAGE_EXPORT_DIRECTORY*>(
    ImageDirectoryEntryToData( base_, TRUE, IMAGE_DIRECTORY_ENTRY_EXPORT, &size ) );

  WORD* ordinals = reinterpret_cast<WORD*>( base_ + directory->AddressOfNameOrdinals );
  DWORD* names = reinterpret_cast<DWORD*>( base_ + directory->AddressOfNames );

  for ( unsigned int i = 0; i < directory->NumberOfFunctions; ++i ) {
    for ( unsigned int k = 0; k < directory->NumberOfNames; ++k ) {
      if ( ordinals[k] == static_cast<WORD>( i ) ) {
        function_name_list_.push_back( reinterpret_cast<char*>( base_ + names[k] ) );
      }
    }
  }
}

TtDynamicLinkLibrary::~TtDynamicLinkLibrary()
{
  FreeLibrary( handle_ );
}


const std::vector<std::string>&
TtDynamicLinkLibrary::GetFunctionNameList( void )
{
  return function_name_list_;
}


FARPROC
TtDynamicLinkLibrary::GetProcessAddress( const char* name )
{
  FARPROC tmp = ::GetProcAddress( handle_, name );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::GetProcAddress ) );
  }
  return tmp;
}
