#include "ttl_define.h"
#include "tt_exception.h"

#include "tt_image_list.h"


TtImageList::TtImageList( HIMAGELIST handle, bool auto_destory ) :
handle_( handle ),
auto_destroy_( auto_destory )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ImageList_Create ) );
  }
}

TtImageList::TtImageList( int width, int height ) :
handle_( ::ImageList_Create( width, height, ILC_COLOR | ILC_MASK, 1, 1 ) ),
auto_destroy_( true )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ImageList_Create ) );
  }
}

TtImageList::TtImageList( int width, int height, unsigned int flags ) :
handle_( ::ImageList_Create( width, height, flags, 1, 1 ) ),
auto_destroy_( true )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ImageList_Create ) );
  }
}

TtImageList::TtImageList( int width, int height, unsigned int flags, int initial, int grow ) :
handle_( ::ImageList_Create( width, height, flags, initial, grow ) ),
auto_destroy_( true )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ImageList_Create ) );
  }
}

TtImageList::~TtImageList()
{
  if ( auto_destroy_ ) {
    this->Destroy();
  }
}

HIMAGELIST
TtImageList::GetHandle( void )
{
  return handle_;
}

bool
TtImageList::GetAutoDestory( void )
{
  return auto_destroy_;
}

void
TtImageList::SetAutoDestory( bool flag )
{
  auto_destroy_ = flag;
}

void
TtImageList::Destroy( void )
{
  if ( handle_ != NULL ) {
    ImageList_Destroy( handle_ );
    handle_ = NULL;
  }
}


void
TtImageList::AddIcon( HICON icon )
{
  if ( ImageList_AddIcon( handle_, icon ) == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ImageList_AddIcon ) );
  }
}

void
TtImageList::AddBitmap( const TtBmpImage& image )
{
  if ( ::ImageList_Add( handle_, image.GetHandle(), NULL ) == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ImageList_Add ) );
  }
}

void
TtImageList::AddBitmap( const TtBmpImage& image, const TtBmpImage& mask )
{
  if ( ::ImageList_Add( handle_, image.GetHandle(), mask.GetHandle() ) == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ImageList_Add ) );
  }
}

void
TtImageList::AddBitmapMasked( const TtBmpImage& image )
{
  this->AddBitmapMasked( image, 0 );
}

void
TtImageList::AddBitmapMasked( const TtBmpImage& image, int mask_color )
{
  if ( ::ImageList_AddMasked( handle_, image.GetHandle(), mask_color ) == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ImageList_AddMasked ) );
  }
}
