#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_temporaly_array.h"

#include "tt_listview.h"


#ifdef MAKELONG
#undef MAKELONG
#endif
#define MAKELONG(a, b) (((LONG)(((WORD)((DWORD_PTR)(a) & 0xffff)))) | (((DWORD)((WORD)((DWORD_PTR)(b) & 0xffff))) << 16))

// -- TtListViewColumn ---------------------------------------------------
TtListViewColumn::TtListViewColumn( TtListView& list_view, int index ) :
list_view_( list_view ),
index_( index )
{
}

void
TtListViewColumn::GetInfo( LVCOLUMN& info ) const
{
  if ( ListView_GetColumn( list_view_.GetHandle(), index_, &info ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_GetColumn ) );
  }
}

void
TtListViewColumn::SetInfo( LVCOLUMN& info )
{
  if ( ListView_SetColumn( list_view_.GetHandle(), index_, &info ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_SetColumn ) );
  }
}


int
TtListViewColumn::GetTextAlign( void ) const
{
  LVCOLUMN tmp;
  tmp.iSubItem = index_;
  tmp.mask = LVCF_FMT;
  this->GetInfo( tmp );
  return tmp.fmt;
}

void
TtListViewColumn::SetTextAlign( int format )
{
  LVCOLUMN tmp;
  tmp.iSubItem = index_;
  tmp.mask = LVCF_FMT;
  tmp.fmt = format;
  this->SetInfo( tmp );
}

int
TtListViewColumn::GetIndex( void ) const
{
  return index_;
}

int
TtListViewColumn::GetWidth( void ) const
{
  int ret = ListView_GetColumnWidth( list_view_.GetHandle(), index_ );
  if ( ret == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_GetColumnWidth ) );
  }
  return ret;
}

void
TtListViewColumn::SetWidth( int width )
{
  if ( ListView_SetColumnWidth( list_view_.GetHandle(), index_, width ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_SetColumnWidth ) );
  }
}

std::string
TtListViewColumn::GetText( void ) const
{
  LVCOLUMN tmp;
  tmp.iSubItem = index_;
  tmp.mask = LVCF_TEXT | LVCF_SUBITEM;
  TtTemporalyString buf( 1024 * 8 );
  tmp.pszText = buf.GetPtr();
  tmp.cchTextMax = buf.GetSize();
  this->GetInfo( tmp );
  return std::string( tmp.pszText );
}

void
TtListViewColumn::SetText( const std::string& text )
{
  LVCOLUMN tmp;
  tmp.iSubItem = index_;
  tmp.mask = LVCF_TEXT | LVCF_SUBITEM;
  tmp.pszText = const_cast<char*>( text.c_str() );
  tmp.cchTextMax = text.size();
  this->SetInfo( tmp );
}


void
TtListViewColumn::Remove( void )
{
  if ( ListView_DeleteColumn( list_view_.GetHandle(), index_ ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_DeleteColumn ) );
  }
}

// -- TtListViewItem -----------------------------------------------------
TtListViewItem::TtListViewItem( TtListView& list_view, int index ) :
list_view_( list_view ),
index_( index )
{
}


void
TtListViewItem::GetInfo( LVITEM& info ) const
{
  if ( ListView_GetItem( list_view_.GetHandle(), &info ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_GetItem ) );
  }
}

void
TtListViewItem::SetInfo( LVITEM& info )
{
  if ( ListView_SetItem( list_view_.GetHandle(), &info ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_SetItem ) );
  }
}


TtListViewItem
TtListViewItem::operator =( const TtListViewItem& obj )
{
  return TtListViewItem( obj.list_view_, obj.index_ );
}


bool
TtListViewItem::GetChecked( void ) const
{
  return ListView_GetCheckState( list_view_.GetHandle(), index_ );
}

void
TtListViewItem::SetChecked( bool flag )
{
  ListView_SetCheckState( list_view_.GetHandle(), index_, flag );
}

bool
TtListViewItem::GetFocused( void ) const
{
  return ListView_GetItemState( list_view_.GetHandle(), index_, LVIS_FOCUSED );
}

void
TtListViewItem::SetFocused( bool flag )
{
  ListView_SetItemState( list_view_.GetHandle(), index_, flag ? LVIS_FOCUSED : 0, LVIS_FOCUSED );
}

bool
TtListViewItem::GetSelected( void ) const
{
  return ListView_GetItemState( list_view_.GetHandle(), index_, LVIS_SELECTED );
}

void
TtListViewItem::SetSelected( bool flag )
{
  ListView_SetItemState( list_view_.GetHandle(), index_, flag ? LVIS_SELECTED : 0, LVIS_SELECTED );
}

void
TtListViewItem::SetSelectionMark( void )
{
  ListView_SetSelectionMark( list_view_.GetHandle(), index_ );
}

std::string
TtListViewItem::GetText( void ) const
{
  return this->GetSubItemText( 0 );
}

void
TtListViewItem::SetText( const std::string& text )
{
  this->SetSubItemText( 0, text );
}

std::string
TtListViewItem::GetSubItemText( int index ) const
{
  TtTemporalyString buf( 1024 * 8 );
  ListView_GetItemText( list_view_.GetHandle(), index_, index, buf.GetPtr(), buf.GetSize() );
  return std::string( buf.GetPtr() );
}

void
TtListViewItem::SetSubItemText( int index, const std::string& text )
{
  ListView_SetItemText( list_view_.GetHandle(), index_, index, const_cast<char*>( text.c_str() ) );
}


int
TtListViewItem::GetIndex( void ) const
{
  return index_;
}

int
TtListViewItem::GetImageIndex( void ) const
{
  LVITEM tmp = {LVIF_IMAGE};
  tmp.iItem = index_;
  this->GetInfo( tmp );
  return tmp.iImage;
}

void
TtListViewItem::SetImageIndex( int index )
{
  LVITEM tmp = {LVIF_IMAGE};
  tmp.iItem = index_;
  tmp.iImage = index;
  this->SetInfo( tmp );
}

void
TtListViewItem::BeginEdit( void )
{
  this->SetFocused( true );
  HWND tmp = ListView_EditLabel( list_view_.GetHandle(), index_ );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_EditLabel ) );
  }
  // tmp eLXgGfBbgRg[Ȃ̂ŉ̂ȁH
}

void
TtListViewItem::Remove( void )
{
  if ( ListView_DeleteItem( list_view_.GetHandle(), index_ ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_DeleteItem ) );
  }
}


// -- TtListView ---------------------------------------------------------
TtListView::TtListView( void ) :
column_count_( 0 )
{
}


int
TtListView::GetView( void )
{
  return this->GetWindowLong( GWL_STYLE ) & LVS_TYPEMASK;
}

void
TtListView::SetView( int view )
{
  this->SetWindowLong( GWL_STYLE, this->GetWindowLong( GWL_STYLE ) | (view & LVS_TYPEMASK) );
}

bool
TtListView::GetHasCheckBox( void )
{
  return ListView_GetExtendedListViewStyle( handle_ ) & LVS_EX_CHECKBOXES;
}

void
TtListView::SetHasCheckBox( bool flag )
{
  ListView_SetExtendedListViewStyle( handle_, ListView_GetExtendedListViewStyle( handle_ ) | (flag ? LVS_EX_CHECKBOXES : 0) );
}

bool
TtListView::GetHasGridLines( void )
{
  return ListView_GetExtendedListViewStyle( handle_ ) & LVS_EX_GRIDLINES;
}

void
TtListView::SetHasGridLines( bool flag )
{
  ListView_SetExtendedListViewStyle( handle_, ListView_GetExtendedListViewStyle( handle_ ) | (flag ? LVS_EX_GRIDLINES : 0) );
}

bool
TtListView::GetLabelEdit( void )
{
  return this->GetWindowLong( GWL_STYLE ) & LVS_EDITLABELS;
}

void
TtListView::SetLabelEdit( bool flag )
{
  this->SetWindowLong( GWL_STYLE, this->GetWindowLong( GWL_STYLE ) | (flag ? LVS_EDITLABELS : 0) );
}

bool
TtListView::GetScrollable( void )
{
  return NOT( this->GetWindowLong( GWL_STYLE ) & LVS_NOSCROLL );
}

void
TtListView::SetScrollable( bool flag )
{
  this->SetWindowLong( GWL_STYLE, this->GetWindowLong( GWL_STYLE ) | (flag ? 0 : LVS_EDITLABELS) );
}

bool
TtListView::GetMultiselect( void )
{
  return NOT( this->GetWindowLong( GWL_STYLE ) & LVS_SINGLESEL );
}

void
TtListView::SetMultiselect( bool flag )
{
  this->SetWindowLong( GWL_STYLE, this->GetWindowLong( GWL_STYLE ) | (flag ? 0 : LVS_SINGLESEL) );
}

bool
TtListView::GetFullRowSelect( void )
{
  return ListView_GetExtendedListViewStyle( handle_ ) & LVS_EX_FULLROWSELECT;
}

void
TtListView::SetFullRowSelect( bool flag )
{
  ListView_SetExtendedListViewStyle( handle_, ListView_GetExtendedListViewStyle( handle_ ) | (flag ? LVS_EX_FULLROWSELECT : 0) );
}

bool
TtListView::GetHideSelection( void )
{
  return NOT( this->GetWindowLong( GWL_STYLE ) & LVS_SHOWSELALWAYS );
}

void
TtListView::SetHideSelection( bool flag )
{
  this->SetWindowLong( GWL_STYLE, this->GetWindowLong( GWL_STYLE ) | (flag ? 0 : LVS_SHOWSELALWAYS) );
}

int
TtListView::GetSorting( void )
{
  return this->GetWindowLong( GWL_STYLE ) & (LVS_SORTASCENDING | LVS_SORTDESCENDING);
}

void
TtListView::SetSorting( int sorting )
{
  this->SetWindowLong( GWL_STYLE, this->GetWindowLong( GWL_STYLE ) | (sorting & (LVS_SORTASCENDING | LVS_SORTDESCENDING)) );
}


int
TtListView::GetHeaderStyle( void )
{
  return this->GetWindowLong( GWL_STYLE ) & (LVS_NOCOLUMNHEADER | LVS_NOSORTHEADER);
}

void
TtListView::SetHeaderStyle( int style )
{
  this->SetWindowLong( GWL_STYLE, this->GetWindowLong( GWL_STYLE ) | (style & (LVS_NOCOLUMNHEADER | LVS_NOSORTHEADER)) );
}

TtImageList
TtListView::GetSmallImageList( void )
{
  HIMAGELIST tmp = ListView_GetImageList( handle_, LVSIL_SMALL );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_GetImageList ) );
  }
  return TtImageList( tmp, false );
}

void
TtListView::SetSmallImageList( TtImageList& image_list )
{
  HIMAGELIST tmp = ListView_SetImageList( handle_, image_list.GetHandle(), LVSIL_SMALL );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_SetImageList ) );
  }
}

TtImageList
TtListView::GetLargeImageList( void )
{
  HIMAGELIST tmp = ListView_GetImageList( handle_, LVSIL_NORMAL );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_GetImageList ) );
  }
  return TtImageList( tmp, false );
}

void
TtListView::SetLargeImageList( TtImageList& image_list )
{
  HIMAGELIST tmp = ListView_SetImageList( handle_, image_list.GetHandle(), LVSIL_NORMAL );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_SetImageList ) );
  }
}

int
TtListView::GetColumnCount( void )
{
  return column_count_;
}

TtListViewColumn
TtListView::MakeNewColumn( void )
{
  LVCOLUMN tmp = {0};
  tmp.iSubItem = column_count_;
  tmp.mask = LVCF_TEXT | LVCF_SUBITEM;
  tmp.pszText = "";
  int index = ListView_InsertColumn( handle_, tmp.iSubItem, &tmp );
  if ( index == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_InsertColumn ) );
  }
  column_count_ += 1;
  return TtListViewColumn( *this, index );
}

TtListViewColumn
TtListView::GetColumn( int index )
{
  return TtListViewColumn( *this, index );
}

int
TtListView::GetItemCount( void )
{
  return ListView_GetItemCount( handle_ );
}

TtListViewItem
TtListView::MakeNewItem( void )
{
  LVITEM tmp = {0};
  tmp.iItem = this->GetItemCount();
  int index = ListView_InsertItem( handle_, &tmp );
  if ( index == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_InsertItem ) );
  }
  return TtListViewItem( *this, index );
}

TtListViewItem
TtListView::GetItem( int index )
{
  return TtListViewItem( *this, index );
}

std::vector<int>
TtListView::GetCheckedIndices( void )
{
  int count = this->GetItemCount();
  std::vector<int> ary;
  for ( int i = 0; i < count; ++i ) {
    if ( ListView_GetCheckState( handle_, i ) ) {
      ary.push_back( i );
    }
  }
  return ary;
}

std::vector<TtListViewItem>
TtListView::GetCheckedItems( void )
{
  int count = this->GetItemCount();
  std::vector<TtListViewItem> ary;
  for ( int i = 0; i < count; ++i ) {
    if ( ListView_GetCheckState( handle_, i ) ) {
      ary.push_back( TtListViewItem( *this, i ) );
    }
  }
  return ary;
}

std::vector<int>
TtListView::GetFocusedIndices( void )
{
  int count = this->GetItemCount();
  std::vector<int> ary;
  for ( int i = 0; i < count; ++i ) {
    if ( ListView_GetItemState( handle_, i, LVIS_FOCUSED ) ) {
      ary.push_back( i );
    }
  }
  return ary;
}

std::vector<TtListViewItem>
TtListView::GetFocusedItems( void )
{
  int count = this->GetItemCount();
  std::vector<TtListViewItem> ary;
  for ( int i = 0; i < count; ++i ) {
    if ( ListView_GetItemState( handle_, i, LVIS_FOCUSED ) ) {
      ary.push_back( TtListViewItem( *this, i ) );
    }
  }
  return ary;
}

int
TtListView::GetSelectedCount( void )
{
  return ListView_GetSelectedCount( handle_ );
}

std::vector<int>
TtListView::GetSelectedIndices( void )
{
  int count = this->GetItemCount();
  std::vector<int> ary;
  for ( int i = 0; i < count; ++i ) {
    if ( ListView_GetItemState( handle_, i, LVIS_SELECTED ) ) {
      ary.push_back( i );
    }
  }
  return ary;
}

std::vector<TtListViewItem>
TtListView::GetSelectedItems( void )
{
  int count = this->GetItemCount();
  std::vector<TtListViewItem> ary;
  for ( int i = 0; i < count; ++i ) {
    if ( ListView_GetItemState( handle_, i, LVIS_SELECTED ) ) {
      ary.push_back( TtListViewItem( *this, i ) );
    }
  }
  return ary;
}


void
TtListView::Arrange( int format )
{
  if ( ListView_Arrange( handle_, format ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_Arrange ) );
  }
}

  void LockDraw( void );
  void UnlockDraw( void );


void
TtListView::LockDraw( void )
{
  this->SendMessage( WM_SETREDRAW, FALSE );
}

void
TtListView::UnlockDraw( void )
{
  this->SendMessage( WM_SETREDRAW, TRUE );
}

void
TtListView::EnsureVisible( int index )
{
  if ( ListView_EnsureVisible( handle_, index, FALSE ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_EnsureVisible ) );
  }
}

void
TtListView::Clear( void )
{
  if ( ListView_DeleteAllItems( handle_ ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( ListView_DeleteAllItems ) );
  }
}
