#include <stdio.h>
#include <errno.h>

#include <sstream>

#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_path.h"
#include "tt_time.h"
#include "tt_util.h"

#include "tt_logger.h"


// -- TtLogger -----------------------------------------------------------
TtLogFlusher
TtLogger::FLUSH;

TtLogger::TtLogger( void ) :
filename_( TtPath::GetExecutingFilePath() + ".log" ),
file_stream_( NULL ),
lock_(),
buffer_()
{
}

TtLogger::TtLogger( const std::string& filename ) :
filename_( filename ),
file_stream_( NULL ),
lock_(),
buffer_()
{
}

TtLogger::~TtLogger()
{
  this->Close();
}

const std::string&
TtLogger::GetFileName( void )
{
  return filename_;
}

void
TtLogger::Open( void )
{
  if ( file_stream_ != NULL ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  file_stream_ = fopen( filename_.c_str(), "a" );
  if ( file_stream_ == NULL ) {
    throw TtFileOpenException( filename_, errno );
  }
}

void
TtLogger::Close( void )
{
  if ( file_stream_ != NULL ) {
    fclose( file_stream_ );
    file_stream_ = NULL;
  }
}

template <>
TtLogger&
TtLogger::operator << <std::string>( const std::string& value )
{
  buffer_.append( value );
  return *this;
}

template <>
TtLogger&
TtLogger::operator << <const char*>( const char* const& value )
{
  buffer_.append( value );
  return *this;
}

void
TtLogger::operator <<( TtLogFlusher& flusher )
{
  NOT_USE_VAR( flusher );
  this->Put( buffer_ );
  buffer_.clear();
}

void
TtLogger::Put( const std::string& value )
{
  std::string tmp;
  this->Prefix( tmp );
  tmp.append( value );
  this->Postfix( tmp );
  if ( fputs( tmp.c_str(), file_stream_ ) == EOF ) {
    throw TT_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( fputs ), errno );
  }
}

void
TtLogger::Prefix( std::string& str )
{
  NOT_USE_VAR( str );
}

void
TtLogger::Postfix( std::string& str )
{
  NOT_USE_VAR( str );
}


// -- TtLoggerStandard ---------------------------------------------------
TtLoggerStandard::TtLoggerStandard( void ) :
TtLogger()
{
}

TtLoggerStandard::TtLoggerStandard( const std::string& filename ) :
TtLogger( filename )
{
}

void
TtLoggerStandard::Prefix( std::string& str )
{
  TtTime now;
  std::ostringstream os;
  os << now.GetYear() << "/" << now.GetMonth() << "/" << now.GetDayOfMonth() << " ";
  os << now.GetHour() << ":" << now.GetMinute() << ":" << now.GetSecond() << " ";
  str.append( os.str() );
}

void
TtLoggerStandard::Postfix( std::string& str )
{
  str.append( "\n" );
}
