#include <windows.h>

#include "ttl_define.h"
#include "tt_exception.h"

#include "tt_menu.h"


// -- TtMenuItem ---------------------------------------------------------
TtMenuItem::TtMenuItem( TtMenu& parent, unsigned int id_or_index, bool has_id ) :
parent_( parent ),
id_or_index_( id_or_index ),
has_id_( has_id )
{
}

TtMenuItem::~TtMenuItem()
{
}

void
TtMenuItem::SetEnabled( bool flag )
{
  ::EnableMenuItem( parent_.GetHandle(), id_or_index_,
                    ( flag ? MF_ENABLED : MF_GRAYED) |
                    (has_id_ ? MF_BYCOMMAND : MF_BYPOSITION) );
}


// -- TtMenu -------------------------------------------------------------
TtMenu::TtMenu( TtWindow& parent ) :
handle_( ::CreateMenu() ),
parent_( parent ),
auto_destroy_( false )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::CreateMenu ) );
  }
}

TtMenu::TtMenu( TtWindow& parent, const std::string& name ) :
handle_( ::LoadMenu( parent.GetInstanceHandle(), name.c_str() ) ),
parent_( parent ),
auto_destroy_( false )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::LoadMenu ) );
  }
}

TtMenu::TtMenu( TtWindow& parent, const std::string& name, bool auto_destroy ) :
handle_( ::LoadMenu( parent.GetInstanceHandle(), name.c_str() ) ),
parent_( parent ),
auto_destroy_( auto_destroy )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::LoadMenu ) );
  }
}


TtMenu::TtMenu( HMENU handle, TtWindow& parent, bool auto_destroy ) :
handle_( handle ),
parent_( parent ),
auto_destroy_( auto_destroy )
{
}


TtMenu::TtMenu( const TtMenu& root, int index ) :
handle_( ::GetSubMenu( root.handle_, index ) ),
parent_( root.parent_ ),
auto_destroy_( false )
{
  if ( handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::GetSubMenu ) );
  }
}

TtMenu::~TtMenu()
{
  if ( auto_destroy_ ) {
    DestroyMenu( handle_ );
  }
}

void
TtMenu::SetAutoDestroy( bool flag )
{
  auto_destroy_ = flag;
}

HMENU
TtMenu::GetHandle( void ) const
{
  return handle_;
}

TtMenuItem
TtMenu::GetItemAt( unsigned int index )
{
  return TtMenuItem( *this, index, false );
}

TtMenuItem
TtMenu::GetItemById( unsigned int id )
{
  return TtMenuItem( *this, id , true );
}

TtMenu
TtMenu::GetSubMenuAt( unsigned int index ) const
{
  return TtMenu( *this, index );
}

void
TtMenu::TrackPopup( int x, int y ) const
{
  this->TrackPopupPrimitive( x, y, TPM_LEFTALIGN | TPM_TOPALIGN );
}

void
TtMenu::TrackPopupRelative( int x, int y ) const
{
  POINT point = {x, y};
  if ( ::ClientToScreen( parent_.GetHandle(), &point ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ClientToScreen ) );
  }
  this->TrackPopup( point.x, point. y );
}

void
TtMenu::TrackPopupPrimitive( int x, int y, int flags ) const
{
  if ( ::TrackPopupMenu( handle_, flags, x, y, 0, parent_.GetHandle(), NULL ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::TrackPopupMenu ) );
  }
}
