#include "tt_thread.h"

#include "tt_message_box.h"


// -- TtMessageBox -------------------------------------------------------
TtMessageBox::TtMessageBox( const std::string& message,
                            const std::string& caption,
                            unsigned int       icon ) :
message_( message ),
caption_( caption ),
icon_( icon ),
default_button_( TtMessageBox::DefautButton::FIRST )
{
}

const std::string&
TtMessageBox::GetMessage( void ) const
{
  return message_;
}

void
TtMessageBox::SetMessage( const std::string& message )
{
  message_.assign( message );
}

void
TtMessageBox::AppendMessage( const std::string& message )
{
  message_.append( message );
}

TtUtil::StringAppender
TtMessageBox::AppendMessage( void )
{
  return TtUtil::StringAppender( message_ );
}

const std::string&
TtMessageBox::GetCaption( void ) const
{
  return caption_;
}

void
TtMessageBox::SetCaption( const std::string& caption )
{
  caption_.assign( caption );
}

unsigned int
TtMessageBox::GetIcon( void ) const
{
  return icon_;
}

void
TtMessageBox::SetIcon( unsigned int icon )
{
  icon_ = icon;
}

unsigned int
TtMessageBox::GetDefaultButton( void ) const
{
  return default_button_;
}

void
TtMessageBox::SetDefaultButton( unsigned int button )
{
  default_button_ = button;
}

unsigned int
TtMessageBox::ShowDialog( void )
{
  return MessageBox( NULL, message_.c_str(), caption_.c_str(),
                     this->GetType() | icon_ | default_button_ );
}



std::map<DWORD, TtMessageBox*>
TtMessageBox::hook_table;

std::map<DWORD, WNDPROC>
TtMessageBox::procedure_table;

LRESULT CALLBACK
TtMessageBox::Hook( int code, WPARAM w_param, LPARAM l_param )
{
  TtMessageBox* box = hook_table[TtThread::GetCurrentThreadId()];
  const CWPRETSTRUCT* p = reinterpret_cast<const CWPRETSTRUCT*>( l_param );

  if ( p->message == WM_NCCREATE ) {
    ::UnhookWindowsHookEx( box->hook_ );
    box->hook_ = NULL;
    procedure_table[TtThread::GetCurrentThreadId()] = reinterpret_cast<WNDPROC>( ::GetWindowLong( p->hwnd, GWL_WNDPROC ) );
    ::SetWindowLong( p->hwnd, GWL_WNDPROC, LONG( Procedure ) );
  }
  return ::CallNextHookEx( box->hook_, code, w_param, l_param );
}

LRESULT CALLBACK
TtMessageBox::Procedure( HWND handle, UINT msg, WPARAM w_param, LPARAM l_param )
{
  TtMessageBox* box  = hook_table[TtThread::GetCurrentThreadId()];
  WNDPROC       proc = procedure_table[TtThread::GetCurrentThreadId()];

  LONG ret = ::CallWindowProc( proc, handle, msg, w_param, l_param );
  if ( msg == WM_INITDIALOG ) {
    ::SetWindowLong( handle, GWL_WNDPROC, LONG( proc ) );
    TtExtraordinarilyWindow( handle, *(box->parent_) ).SetCenterRelativeToParent();
  }
  return ret;
}

unsigned int
TtMessageBox::ShowDialog( TtWindow& parent )
{
  hook_ = ::SetWindowsHookEx( WH_CALLWNDPROCRET,
                              Hook,
                              ::GetModuleHandle( NULL ),
                              TtThread::GetCurrentThreadId() );
  parent_ = &parent;
  hook_table[TtThread::GetCurrentThreadId()] = this;
  return MessageBox( parent.GetHandle(), message_.c_str(), caption_.c_str(),
                     this->GetType() | icon_ | default_button_ );
}

#ifdef DEBUG
// ev[g̖IȃCX^X肭Ȃ̂ő
namespace {
  TtMessageBoxOk               ok;
  TtMessageBoxOkCancel         ok_cancel;
  TtMessageBoxRetryCancel      retry_cancel;
  TtMessageBoxYesNo            yes_no;
  TtMessageBoxYesNoCancel      yes_no_cancel;
  TtMessageBoxAbortRetryIgnore abort_retry_ignore;
}
#endif
