#pragma comment( lib, "psdk\shlwapi.lib" )

#include <stdio.h>
#include <shlwapi.h>

#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_temporaly_array.h"

#include "tt_path.h"


// pX̔E
bool
TtPath::IsDirectory( const std::string& path )
{
  return ::PathIsDirectory( path.c_str() );
}

bool
TtPath::IsFileSpec( const std::string& path )
{
  return ::PathIsFileSpec( path.c_str() );
}

bool
TtPath::IsRelative( const std::string& path )
{
  return ::PathIsRelative( path.c_str() );
}

bool
TtPath::IsRoot( const std::string& path )
{
  return ::PathIsRoot( path.c_str() );
}

bool
TtPath::MatchSpec( const std::string& path, const std::string& match )
{
  return ::PathMatchSpec( path.c_str(), match.c_str() );
}

bool
TtPath::FileExists( const std::string& path )
{
  return ::PathFileExists( path.c_str() );
}

// Ȃ
std::string
TtPath::FindFileName( const std::string& path )
{
  return ::PathFindFileName( path.c_str() );
}

std::string
TtPath::FindExtension( const std::string& path )
{
  return ::PathFindExtension( path.c_str() );
}

// ҏWȂ
std::string
TtPath::EditPath( const std::string& path, bool f_drive, bool f_dir, bool f_fname, bool f_ext )
{
  char drive[MAX_PATH], dir[MAX_PATH], fname[MAX_PATH], ext[MAX_PATH];
  _splitpath( path.c_str(), drive, dir, fname, ext );

  std::string tmp;
  tmp.append( f_drive ? drive : "" );
  tmp.append( f_dir   ? dir   : "" );
  tmp.append( f_fname ? fname : "" );
  tmp.append( f_ext   ? ext   : "" );
  return tmp;
}

std::string
TtPath::DirName( const std::string& path )
{
  std::string tmp = TtPath::EditPath( path, true, true, false, false );
  if ( tmp.size() == 0 ) {
    tmp = ".";
  }
  else if ( tmp[tmp.size() - 1] == '\\' || tmp[tmp.size() - 1] == '\/' ) {
    tmp.erase( tmp.size() - 1 );
  }
  return tmp;
}

std::string
TtPath::BaseName( const std::string& path )
{
  return TtPath::EditPath( path, false, false, true, true );
}

std::string
TtPath::CompactPath( const std::string& src, unsigned int max )
{
  TtTemporalyString buf( max + 1 );
  if ( ::PathCompactPathEx( buf.GetPtr(), src.c_str(), max, '\\' ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::PathCompactPathEx ) );
  }
  return std::string( buf.GetPtr() );
}

std::string
TtPath::QuoteIfHasSpaces( const std::string& path )
{
  TtTemporalyString buf( ( path.size() > MAX_PATH ? path.size() : MAX_PATH ) + 10 );
  path.copy( buf.GetPtr(), path.size() );
  buf.GetPtr()[path.size()] = '\0';
  ::PathQuoteSpaces( buf.GetPtr() );
  return std::string( buf.GetPtr() );
}

std::string
TtPath::ChangeExtension( const std::string& path, const std::string& ext )
{
  return TtPath::EditPath( path, true, true, true, false ) + "." + ext;
}


// 擾Ȃ
std::string
TtPath::GetExecutingFilePath( void )
{
  char buf[MAX_PATH + 1];
  ::GetModuleFileName( NULL, buf, sizeof( buf ) );
  return std::string( buf );
}

std::string
TtPath::GetExecutingFilePathWithoutExtension( void )
{
  return TtPath::EditPath( TtPath::GetExecutingFilePath(), true, true, true, false );
}

std::string
TtPath::GetExecutingFilePathCustomExtension( const std::string& extension )
{
  std::string buf = TtPath::GetExecutingFilePathWithoutExtension();
  buf.append( "." );
  buf.append( extension );
  return buf;
}


std::string
TtPath::GetExpandPath( const std::string& path )
{
  TtTemporalyString buf( ( path.size() > MAX_PATH ? path.size() : MAX_PATH ) + 1 );
  PathSearchAndQualify( path.c_str(), buf.GetPtr(), buf.GetSize() );
  return std::string( buf.GetPtr() );
}
