#include "ttl_define.h"

#include "tt_temporaly_array.h"
#include "tt_thread.h"
#include "tt_exception.h"

#include "tt_property_sheet.h"


// -- TtPropertySheetPage ------------------------------------------------
BOOL CALLBACK
TtPropertySheetPage::WindowProcedureForTTLPropertySheetPage( HWND handle, UINT msg, WPARAM w_param, LPARAM l_param )
{
  if ( msg == WM_INITDIALOG ) {
    TtPropertySheetPage& page = *(reinterpret_cast<TtPropertySheetPage*>(
      reinterpret_cast<LPPROPSHEETPAGE>( l_param )->lParam ));
    page.handle_ = handle;
    page.SetFont( TtFont::DEFAULT );
    TtWindow::GetTtWindowTable().insert( std::pair<HWND, TtWindow*>( handle, &page ) );
    if ( NOT( page.Created() ) ) {
      ::DestroyWindow( page.handle_ );
      return FALSE;
    }
  }
  TtWindow* window = TtWindow::FindWindow( handle );
  return ( window != NULL ) ? window->WindowProcedure( msg, w_param, l_param ) : FALSE;
}


UINT CALLBACK
TtPropertySheetPage::ProcedureForTTLPropertySheetPage( HWND handle, UINT msg, LPPROPSHEETPAGE ppsp )
{
  NOT_USE_VAR( handle );

  TtPropertySheetPage& page = *(reinterpret_cast<TtPropertySheetPage*>( ppsp->lParam ));
  switch ( msg ) {
  case PSPCB_CREATE:
    return page.AtPageCreate();

  case PSPCB_RELEASE:
    page.AtPageRelease();
    if ( page.handle_ != NULL ) {
      TtWindow::GetTtWindowTable().erase( page.handle_ );
      page.handle_ = NULL;
    }
    break;
  }
  return 0;
}

DWORD
TtPropertySheetPage::GetStyle( void )
{
  return WS_DLGFRAME;
}

DWORD
TtPropertySheetPage::GetExtendedStyle( void )
{
  return 0;
}


bool
TtPropertySheetPage::Created( void )
{
  return true;
}

bool
TtPropertySheetPage::AtPageCreate( void )
{
  return true;
}

void
TtPropertySheetPage::AtPageRelease( void )
{
}


LRESULT
TtPropertySheetPage::DefaultWindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param )
{
  NOT_USE_VAR( msg );
  NOT_USE_VAR( w_param );
  NOT_USE_VAR( l_param );
  return FALSE;
}

LRESULT
TtPropertySheetPage::WMDestroyReceived( void )
{
  return FALSE;
}

LRESULT
TtPropertySheetPage::WMNotifyReceived( int id, int code, HWND child )
{
  switch ( code ) {
  case PSN_SETACTIVE:
    this->AtSetActive();
    return TRUE;

  case PSN_KILLACTIVE:
    if ( NOT( this->AtKillActive() ) ) {
      this->SetWindowLong( DWL_MSGRESULT, TRUE );
    }
    return TRUE;

  case PSN_APPLY:
    if ( NOT( this->AtApply() ) ) {
      this->SetWindowLong( DWL_MSGRESULT, TRUE );
    }
    return TRUE;

  case PSN_RESET:
    this->AtReset();
    return TRUE;
  }
  return this->TtForm::WMNotifyReceived( id, code, child );
}


void
TtPropertySheetPage::AtSetActive( void )
{
}

bool
TtPropertySheetPage::AtKillActive( void )
{
  return true;
}

bool
TtPropertySheetPage::AtApply( void )
{
  return true;
}

void
TtPropertySheetPage::AtReset( void )
{
}


LRESULT
TtPropertySheetPage::WindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param )
{
  LRESULT result = this->TtForm::WindowProcedure( msg, w_param, l_param );
  return ( result == 0 || result == TtWindow::MSG_DEFAULT ) ? FALSE : TRUE;
}


TtPropertySheetPage::TtPropertySheetPage( TtPropertySheet& sheet, const std::string& title ) :
page_(),
title_( title ),
template_object_( GlobalAlloc( GMEM_FIXED | GMEM_ZEROINIT, sizeof( DLGTEMPLATE ) + 10 ) )
{
  parent_ = &sheet;

  page_.dwSize      = sizeof( PROPSHEETPAGE );
  page_.dwFlags     = PSP_DEFAULT | PSP_DLGINDIRECT | PSP_USECALLBACK | PSP_USETITLE;
  page_.pResource   = static_cast<const DLGTEMPLATE*>( template_object_ );
  page_.pszTitle    = title_.c_str();
  page_.pfnDlgProc  = TtPropertySheetPage::WindowProcedureForTTLPropertySheetPage;
  page_.lParam      = reinterpret_cast<long>( this );
  page_.pfnCallback = TtPropertySheetPage::ProcedureForTTLPropertySheetPage;

  DLGTEMPLATE& dialog_template = *(static_cast<DLGTEMPLATE*>( template_object_ ) );
  dialog_template.style = this->GetStyle();
  dialog_template.dwExtendedStyle = this->GetExtendedStyle();
  dialog_template.cdit = 0;
  dialog_template.x = 0;
  dialog_template.y = 0;
  dialog_template.cx = 0;
  dialog_template.cy = 0;

  sheet.pages_.push_back( this );
}

void
TtPropertySheetPage::PresetSizeAtPageCreate( int width, int height )
{
  DLGTEMPLATE& dialog_template = *(static_cast<DLGTEMPLATE*>( template_object_ ) );
  dialog_template.cx = static_cast<short>( width );
  dialog_template.cy = static_cast<short>( height );
}


TtPropertySheetPage::~TtPropertySheetPage()
{
  GlobalFree( template_object_ );
}


const std::string&
TtPropertySheetPage::GetTitle( void ) const
{
  return title_;
}

void
TtPropertySheetPage::CreatePrimitive( CreateParameter& parameter )
{
  NOT_USE_VAR( parameter );
  throw TtInvalidOperationException( typeid( *this ) );
}

void
TtPropertySheetPage::TellChanged( void )
{
  PropSheet_Changed( parent_->GetHandle(), handle_ );
}


// -- TtPropertySheet ----------------------------------------------------
std::map<DWORD, TtPropertySheet*>
TtPropertySheet::TABLE;

int CALLBACK
TtPropertySheet::ProcedureForTTLPropertySheet( HWND handle, UINT msg, LPARAM l_param )
{
  NOT_USE_VAR( handle );
  NOT_USE_VAR( l_param );

  switch ( msg ) {
  case PSCB_INITIALIZED: {
    if ( TABLE.find( TtThread::GetCurrentThreadId() ) == TABLE.end() ) {
      throw TT_INTERNAL_EXCEPTION;
    }
    TtPropertySheet& sheet = *TABLE[TtThread::GetCurrentThreadId()];
    sheet.handle_ = handle;
    sheet.SetWindowLong( GWL_EXSTYLE, sheet.GetWindowLong( GWL_EXSTYLE ) & ~WS_EX_CONTEXTHELP );
    if ( NOT( sheet.Created() ) ) {
      ::DestroyWindow( sheet.handle_ );
      return 0;
    }
    break;
  }

  case PSCB_PRECREATE:
    break;
  }
  return 0;
}



TtPropertySheet::TtPropertySheet( bool has_apply_button ) :
pages_(),
has_apply_button_( has_apply_button )
{
}


void
TtPropertySheet::CreatePrimitive( CreateParameter& parameter )
{
  NOT_USE_VAR( parameter );
  throw TtInvalidOperationException( typeid( *this ) );
}


void
TtPropertySheet::SetHasApplyButton( bool has_apply_button )
{
  has_apply_button_ = has_apply_button;
}

bool
TtPropertySheet::GetHasApplyButton( void ) const
{
  return has_apply_button_;
}


bool
TtPropertySheet::ShowDialog( TtForm& parent )
{
  parent_ = &parent;
  instance_handle_ = parent.GetInstanceHandle();
  TtTemporalyArray<PROPSHEETPAGE> page_array( pages_.size() );
  unsigned int page_count = 0;

  for ( unsigned int i = 0; i < pages_.size(); ++i ) {
    if ( pages_[i]->AtPageCreate() ) {
      pages_[i]->page_.hInstance = instance_handle_;
      page_array.GetPtr()[i] = pages_[i]->page_;
      ++page_count;
    }
  }

  PROPSHEETHEADER sheet;
  sheet.dwSize      = sizeof( PROPSHEETHEADER );
  sheet.dwFlags     = ( PSH_DEFAULT | 0x10 | PSH_USECALLBACK | PSH_PROPSHEETPAGE |
                        (has_apply_button_ ? 0 : PSH_NOAPPLYNOW) );
  sheet.hwndParent  = parent.GetHandle();
  sheet.hInstance   = instance_handle_;
  sheet.pszCaption  = "";
  sheet.nPages      = page_count;
  sheet.nStartPage  = 0;
  sheet.ppsp        = page_array.GetPtr();
  sheet.pfnCallback = TtPropertySheet::ProcedureForTTLPropertySheet;

  TABLE[TtThread::GetCurrentThreadId()] = this;
  int ret = ::PropertySheet( &sheet );
  TABLE.erase( TtThread::GetCurrentThreadId() );
  if ( ret == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::PropertySheet ) );
  }
  return ret == ID_PSREBOOTSYSTEM || ret == ID_PSRESTARTWINDOWS;
}
