#ifndef TT_SQUIRREL_H_
#define TT_SQUIRREL_H_

#include <string>

#include "squirrel.h"

#include "tt_squirrel_object_type.h"

namespace TtSquirrel {
  class VirtualMachine {
  public:
    // -- Stack ------------------------------------------------------------
    class Stack {
      friend VirtualMachine;

    public:
      static const int Top  = -1;
      static const int Base = 1;

    private:
      explicit Stack( HSQUIRRELVM& vm );

    public:
      void PushRootTable( void );
      void Push( int n );
      void Push( float f );
      void Push( const char* str );
      void Push( const std::string& str );
      void Push( bool flag );
      void Push( void* ptr );
      void PushNull( void );

      ObjectType GetType( int index );

      int         GetAsInteger( int index );
      float       GetAsFloat( int index );
      std::string GetAsString( int index );
      bool        GetAsBoolean( int index );
      void*       GetAsUserPointer( int index );

      void PopTop( void );
      void Pop( int count );

      void MoveToTop( int index );

      void Remove( int index );

      void Get( int index );
      void Set( int index );

      int GetTopIndex( void );

      void Call( int param_count, bool need_return_value = true, bool raise_exception_in_error = true );

    private:
      HSQUIRRELVM& vm_;
    };

    // -- VirtualMachine -------------------------------------------------
    explicit VirtualMachine( int stack_size = 1024 );
    ~VirtualMachine();

    HSQUIRRELVM GetHandle( void );

    Stack GetStack( void );

  private:
    HSQUIRRELVM vm_;
  };

  void ResultTest( SQRESULT result, const char* function_name );
  void ObjectTypeCheck( ObjectType require, ObjectType actual );

  void PrintFunc( HSQUIRRELVM v, const SQChar* s, ... );
}

#endif // TT_SQUIRREL_H_
